@@ -23,10 +23,7 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 #  USA
 
-from __future__ import absolute_import, print_function
-
 import apt_pkg
-import apt
 import copy
 from hashlib import md5
 import re
@@ -60,23 +57,16 @@ import aptsources.distro
 import softwareproperties
 
 from .AptAuth import AptAuth
-from aptsources.sourceslist import SourcesList, SourceEntry
-from . import shortcuts
-from . import ppa
-from . import cloudarchive
+from aptsources.sourceslist import (SourcesList, SourceEntry)
+from softwareproperties.shortcuthandler import InvalidShortcutException
+from softwareproperties.shortcuts import shortcut_handler
 
 from gi.repository import Gio
 
-_SHORTCUT_FACTORIES = [
-    ppa.shortcut_handler,
-    cloudarchive.shortcut_handler,
-    shortcuts.shortcut_handler,
-]
-
 
 class SoftwareProperties(object):
 
-  # known (whitelisted) channels
+  # directory containing channels
   CHANNEL_PATH="/usr/share/app-install/channels/"
 
   # release upgrades policy
@@ -94,8 +84,8 @@ class SoftwareProperties(object):
   }
 
   def __init__(self, datadir=None, options=None, rootdir="/"):
-    """ Provides the core functionality to configure the used software 
-        repositories, the corresponding authentication keys and 
+    """ Provides the core functionality to configure the used software
+        repositories, the corresponding authentication keys and
         update automation """
     self.popconfile = rootdir+"/etc/popularity-contest.conf"
 
@@ -111,7 +101,7 @@ class SoftwareProperties(object):
 
     self.sourceslist = SourcesList()
     self.distro = aptsources.distro.get_distro()
-    
+
     self.seen_server = []
     self.modified_sourceslist = False
 
@@ -155,8 +145,8 @@ class SoftwareProperties(object):
     self.write_config()
 
   def get_update_automation_level(self):
-    """ Parse the apt cron configuration. Try to fit a predefined use case 
-        and return it. Special case: if the user made a custom 
+    """ Parse the apt cron configuration. Try to fit a predefined use case
+        and return it. Special case: if the user made a custom
         configurtation, that we cannot represent it will return None """
     if apt_pkg.config.find_i(softwareproperties.CONF_MAP["autoupdate"]) > 0:
         # Autodownload
@@ -178,9 +168,9 @@ class SoftwareProperties(object):
         return None
 
   def set_update_automation_level(self, state):
-    """ Set the apt periodic configurtation to the selected 
-        update automation level. To synchronize the cache update and the 
-        actual upgrading function, the upgrade function, e.g. unattended, 
+    """ Set the apt periodic configurtation to the selected
+        update automation level. To synchronize the cache update and the
+        actual upgrading function, the upgrade function, e.g. unattended,
         will run every day, if enabled. """
     if state == softwareproperties.UPDATE_INST_SEC:
         apt_pkg.config.set(softwareproperties.CONF_MAP["unattended"], str(1))
@@ -236,39 +226,36 @@ class SoftwareProperties(object):
      """
     # we are note using ConfigParser.write() as it removes comments
     if not os.path.exists(self.RELEASE_UPGRADES_CONF):
-      f = open(self.RELEASE_UPGRADES_CONF,"w")
-      f.write("[DEFAULT]\nPrompt=%s\n"% self.release_upgrades_policy_map[i])
+      with open(self.RELEASE_UPGRADES_CONF,"w") as f:
+        f.write("[DEFAULT]\nPrompt=%s\n"% self.release_upgrades_policy_map[i])
       return True
-    f = open(self.RELEASE_UPGRADES_CONF,"r")
-    out = NamedTemporaryFile(mode="w+")
-    for line in f:
-      line = line.strip()
-      if line.lower().startswith("prompt"):
-        out.write("Prompt=%s\n" % self.release_upgrades_policy_map[i])
-      else:
-        out.write(line+"\n")
-    out.flush()
-    shutil.copymode(self.RELEASE_UPGRADES_CONF, out.name)
-    shutil.copy(out.name, self.RELEASE_UPGRADES_CONF)
+    with open(self.RELEASE_UPGRADES_CONF,"r") as f, NamedTemporaryFile(mode="w+") as out:
+      for line in f:
+        line = line.strip()
+        if line.lower().startswith("prompt"):
+          out.write("Prompt=%s\n" % self.release_upgrades_policy_map[i])
+        else:
+          out.write(line+"\n")
+      out.flush()
+      shutil.copymode(self.RELEASE_UPGRADES_CONF, out.name)
+      shutil.copy(out.name, self.RELEASE_UPGRADES_CONF)
     return True
 
   def get_popcon_participation(self):
-    """ Will return True if the user wants to participate in the popularity 
-        contest. Otherwise it will return False. Special case: if no 
+    """ Will return True if the user wants to participate in the popularity
+        contest. Otherwise it will return False. Special case: if no
         popcon is installed it will return False """
     if os.path.exists(self.popconfile):
-        lines = open(self.popconfile).read().split("\n")
-        active = False
+        with open(self.popconfile) as f:
+          lines = f.read().split("\n")
         for line in lines:
             try:
                 (key,value) = line.split("=")
                 if key == "PARTICIPATE" and value.strip('"').lower() == "yes":
-                    active = True
+                    return True
             except ValueError:
                 continue
-        return active
-    else:
-        return False
+    return False
 
   def set_popcon_pariticipation(self, is_helpful):
     """ Enable or disable the participation in the popularity contest """
@@ -285,19 +272,21 @@ class SoftwareProperties(object):
     else:
         # create a new popcon config file
         m = md5()
-        m.update(open("/dev/urandom", "rb").read(1024))
+        with open("/dev/urandom", "rb") as f:
+            m.update(f.read(1024))
         id = m.hexdigest()
         lines = []
         lines.append("MY_HOSTID=\"%s\"\n" % id)
         lines.append("PARTICIPATE=\"%s\"\n" % str(value))
         lines.append("USE_HTTP=\"yes\"\n")
-    open(self.popconfile, "w").writelines(lines)
+    with open(self.popconfile, "w") as f:
+        f.writelines(lines)
 
   def get_source_code_state(self):
-    """Return True if all distro componets are also available as 
+    """Return True if all distro componets are also available as
        source code. Otherwise return Flase. Special case: If the
        configuration cannot be represented return None"""
-  
+
     if len(self.distro.source_code_sources) < 1:
         # we don't have any source code sources, so
         # uncheck the button
@@ -346,7 +335,7 @@ class SoftwareProperties(object):
     print("\n")
 
   def massive_debug_output(self):
-    """Print the complete sources.list""" 
+    """Print the complete sources.list"""
     print("START SOURCES.LIST:")
     for source in self.sourceslist:
         print(source.str())
@@ -361,12 +350,12 @@ class SoftwareProperties(object):
 
   def enable_component(self, comp):
     """Enable a component of the distro"""
-    self.distro.enable_component(comp) 
+    self.distro.enable_component(comp)
     self.set_modified_sourceslist()
 
   def disable_component(self, comp):
     """Disable a component of the distro"""
-    self.distro.disable_component(comp) 
+    self.distro.disable_component(comp)
     self.set_modified_sourceslist()
 
   def _find_template_from_string(self, name):
@@ -410,7 +399,7 @@ class SoftwareProperties(object):
     for source in self.distro.source_code_sources:
         self.sourceslist.remove(source)
     self.set_modified_sourceslist()
-  
+
   def enable_source_code_sources(self):
     """Enable source code source for all distro sources"""
     sources = []
@@ -445,7 +434,7 @@ class SoftwareProperties(object):
     for source in self.sourceslist.list:
         source_bkp = SourceEntry(line=source.line,file=source.file)
         self.sourceslist_backup.append(source_bkp)
-  
+
   def _find_source_from_string(self, line):
     # ensure that we have a current list, it might have been changed underneath
     # us
@@ -475,7 +464,7 @@ class SoftwareProperties(object):
     return False
 
   def revert(self):
-    """Revert all settings to the state when software-properties 
+    """Revert all settings to the state when software-properties
        was launched"""
     #FIXME: GPG keys are still missing
     self.restore_apt_conf()
@@ -578,7 +567,7 @@ class SoftwareProperties(object):
   def get_cdrom_sources(self):
     """Return the list of CDROM based distro sources"""
     return self.distro.cdrom_sources
-      
+
   def get_comp_download_state(self, comp):
     """Return a tuple: the first value describes if a component is enabled
        in the Internet repositories. The second value describes if the
@@ -588,7 +577,7 @@ class SoftwareProperties(object):
 
   def get_comp_child_state(self, template):
     """Return a tuple: the first value describes if a component is enabled
-       in one of the child source that matcth the given template. 
+       in one of the child source that matcth the given template.
        The second value describes if the first value is inconsistent."""
     comps = []
     for child in self.distro.child_sources:
@@ -600,17 +589,17 @@ class SoftwareProperties(object):
         return (True, False)
     elif len(comps) > 0 and\
         len(self.distro.enabled_comps ^ set(comps)) != 0:
-        # A matching child source does exist but doesn't include all 
+        # A matching child source does exist but doesn't include all
         # enabled distro components
         return(False, True)
     else:
         # There is no corresponding child source at all
         return (False, False)
-  
+
   def reload_sourceslist(self):
     self.sourceslist.refresh()
     self.sourceslist_visible=[]
-    self.distro.get_sources(self.sourceslist)    
+    self.distro.get_sources(self.sourceslist)
 
   def write_config(self):
     """Write the current apt configuration to file"""
@@ -625,7 +614,8 @@ class SoftwareProperties(object):
         break
     else:
       print("No config found, creating one")
-      open(conffiles[0], "w")
+      with open(conffiles[0], "w") as f:
+        f.write("")
 
     # ensure /etc/cron.daily/apt is executable
     ac = "/etc/cron.daily/apt"
@@ -640,77 +630,57 @@ class SoftwareProperties(object):
       # read the old content first
       content = []
       if os.path.isfile(periodic):
-        content = open(periodic, "r").readlines()
+        with open(periodic, "r") as f:
+          content = f.readlines()
         cnf = apt_pkg.config.subtree("APT::Periodic")
 
         # then write a new file without the updated keys
-        f = open(periodic, "w")
-        for line in content:
-          for key in cnf.list():
-            if line.find("APT::Periodic::%s" % (key)) >= 0:
-              break
-          else:
-            f.write(line)
+        with open(periodic, "w") as f:
+          for line in content:
+            for key in cnf.list():
+              if line.find("APT::Periodic::%s" % (key)) >= 0:
+                break
+            else:
+              f.write(line)
 
-        # and append the updated keys
-        for i in cnf.list():
-          f.write("APT::Periodic::%s \"%s\";\n" % (i, cnf.find_i(i)))
-        f.close()    
+          # and append the updated keys
+          for i in cnf.list():
+            f.write("APT::Periodic::%s \"%s\";\n" % (i, cnf.find_i(i)))
 
   def save_sourceslist(self):
-    """Backup the existing sources.list files and write the current 
+    """Backup the existing sources.list files and write the current
        configuration"""
     self.sourceslist.backup(".save")
     self.sourceslist.save()
 
-  def _is_line_in_whitelisted_channel(self, srcline):
+  def _get_channel_from_line(self, srcline):
     """
     helper that checks if a given line is in the source list
     return the channel name or None if not found
     """
     srcentry = SourceEntry(srcline)
     if os.path.exists(self.CHANNEL_PATH):
-      for f in glob.glob("%s/*.list" % self.CHANNEL_PATH):
-        for line in open(f):
-          if line.strip().startswith("#"):
-            continue
-          if srcentry == SourceEntry(line):
-            return os.path.splitext(os.path.basename(f))[0]
+      for filename in glob.glob("%s/*.list" % self.CHANNEL_PATH):
+        with open(filename) as f:
+          for line in f:
+            if line.strip().startswith("#"):
+              continue
+            if srcentry == SourceEntry(line):
+              return os.path.splitext(os.path.basename(filename))[0]
     return None
 
-  def check_and_add_key_for_whitelisted_channels(self, srcline):
-    # This is maintained for any legacy callers
-    return self.check_and_add_key_for_whitelisted_shortcut(shortcut_handler(srcline))
-
-  def check_and_add_key_for_whitelisted_shortcut(self, shortcut):
+  def check_and_add_key_for_channel_shortcut(self, shortcut):
     """
     helper that adds the gpg key of the channel to the apt
-    keyring *if* the channel is in the whitelist
-    /usr/share/app-install/channels or it is a public Launchpad PPA.
+    keyring *if* the channel is in /usr/share/app-install/channels
     """
-    (srcline, _fname) = shortcut.expand(
-        codename=self.distro.codename, distro=self.distro.id.lower())
-    channel = self._is_line_in_whitelisted_channel(srcline)
+    srcline = shortcut.SourceEntry().line
+    channel = self._get_channel_from_line(srcline)
     if channel:
       keyp = "%s/%s.key" % (self.CHANNEL_PATH, channel)
       self.add_key(keyp)
-
-    cdata = (shortcut.add_key, {'keyserver': (self.options)})
-    def addkey_func():
-        func, kwargs = cdata
-        msg = "Added key."
-        try:
-            ret = func(**kwargs)
-            if not ret:
-                msg = "Failed to add key."
-        except Exception as e:
-            ret = False
-            msg = str(e)
-        self.myqueue.put([ret, msg])
-
-    worker = threading.Thread(target=addkey_func)
-    worker.start()
-    return worker
+      return True
+    return False
 
   def update_interface(self):
     " abstract interface to keep the UI alive "
@@ -736,18 +706,21 @@ class SoftwareProperties(object):
     """
     Add a source for the given line.
     """
-    return self.add_source_from_shortcut(
-        shortcut=shortcut_handler(line.strip()),
-        enable_source_code=enable_source_code)
+    try:
+        shortcut = shortcut_handler(line.strip())
+    except InvalidShortcutException:
+      return False
+
+    return self.add_source_from_shortcut(shortcut, enable_source_code)
 
   def add_source_from_shortcut(self, shortcut, enable_source_code=False):
     """
     Add a source with the given shortcut and add the signing key if the
-    site is in whitelist or the shortcut implementer adds it.
+    site is a known channel or the shortcut implementer adds it.
     """
 
-    (deb_line, file) = shortcut.expand(
-        codename=self.distro.codename, distro=self.distro.id.lower())
+    deb_line = shortcut.SourceEntry().line
+    file = shortcut.sourceparts_file
     deb_line = self.expand_http_line(deb_line)
     debsrc_entry_type = 'deb-src' if enable_source_code else '# deb-src'
     debsrc_line = debsrc_entry_type + deb_line[3:]
@@ -755,7 +728,8 @@ class SoftwareProperties(object):
     new_debsrc_entry = SourceEntry(debsrc_line, file)
     if new_deb_entry.invalid or new_debsrc_entry.invalid:
       return False
-    worker = self.check_and_add_key_for_whitelisted_shortcut(shortcut)
+    if not self.check_and_add_key_for_channel_shortcut(shortcut):
+      shortcut.add_key()
     self.sourceslist.add(new_deb_entry.type,
                          new_deb_entry.uri,
                          new_deb_entry.dist,
@@ -771,16 +745,6 @@ class SoftwareProperties(object):
                          file=new_debsrc_entry.file,
                          architectures=new_debsrc_entry.architectures)
     self.set_modified_sourceslist()
-    if worker:
-        # wait for GPG key to be downloaded
-        worker.join(30)
-        if worker.isAlive():
-            # thread timed out.
-            raise shortcuts.ShortcutException("Error: retrieving gpg key timed out.")
-        result, msg = self.myqueue.get()
-        if not result:
-            raise shortcuts.ShortcutException(msg)
-
     if self.options and self.options.update:
         import apt
         cache = apt.Cache()
@@ -860,33 +824,37 @@ class SoftwareProperties(object):
     except:
       return False
 
-  def get_package_id(self, ver):
+  def get_package_id(self, apt_cache, pkg):
     """ Return the PackageKit package id """
-    assert isinstance(ver, apt.package.Version)
-    return "%s;%s;%s;" % (ver.package.shortname, ver.version,
-                          ver.package.architecture())
-
+    assert isinstance(pkg, apt_pkg.Package)
+    cur_ver = pkg.current_ver
+    if cur_ver:
+        ver = cur_ver.ver_str
+        arch = cur_ver.arch
+    else:
+        depcache = apt_pkg.DepCache(apt_cache)
+        candidate = depcache.get_candidate_ver(pkg)
+        ver = candidate.ver_str
+        arch = candidate.arch
+    return "%s;%s;%s;" % (pkg.name, ver, arch)
 
   @staticmethod
-  def get_dependencies(apt_cache, package_name, pattern=None):
+  def get_dependencies(apt_cache, package, pattern=None):
     """ Get the package dependencies, which can be filtered out by a pattern """
+    depcache = apt_pkg.DepCache(apt_cache)
+    candidate = depcache.get_candidate_ver(package)
+
     dependencies = []
-    for or_group in apt_cache[package_name].candidate.dependencies:
-      for dep in or_group:
-        if dep.rawtype in ["Depends", "PreDepends"]:
-          dependencies.append(dep.name)
-    if pattern:
-      dependencies = [ x for x in dependencies if x.find(pattern) != -1 ]
+    try:
+        for dep_list in candidate.depends_list_str.get('Depends'):
+            for dep_name, dep_ver, dep_op in dep_list:
+                if dep_name.find(pattern) != -1:
+                    dependencies.append(apt_cache[dep_name])
+    except (KeyError, TypeError):
+        return []
+
     return dependencies
 
-def shortcut_handler(shortcut):
-    for factory in _SHORTCUT_FACTORIES:
-        ret = factory(shortcut)
-        if ret is not None:
-            return ret
-
-    raise shortcuts.ShortcutException("Unable to handle input '%s'" % shortcut)
-
 
 if __name__ == "__main__":
   sp = SoftwareProperties()
