@@ -1,26 +1,28 @@
 # dialog_add.py.in - dialog to add a new repository
-#  
+#
 #  Copyright (c) 2006 FSF Europe
-#              
-#  Authors: 
+#
+#  Authors:
 #       Sebastian Heinlein <glatzor@ubuntu.com>
 #
-#  This program is free software; you can redistribute it and/or 
-#  modify it under the terms of the GNU General Public License as 
+#  This program is free software; you can redistribute it and/or
+#  modify it under the terms of the GNU General Public License as
 #  published by the Free Software Foundation; either version 2 of the
 #  License, or (at your option) any later version.
-# 
+#
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU General Public License for more details.
-# 
+#
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 #  USA
 
 import os
+import gi
+gi.require_version("Gtk", "3.0")
 from gi.repository import GObject, Gtk
 from gettext import gettext as _
 import threading
@@ -83,10 +85,10 @@ class DialogMirror:
     self.country_info = CountryInformation()
 
     setup_ui(self, os.path.join(datadir, "gtkbuilder", "dialog-mirror.ui"), domain="software-properties")
-    
+
     self.dialog = self.dialog_mirror
     self.dialog.set_transient_for(parent)
-    
+
     self.dialog_test = self.dialog_mirror_test
     self.dialog_test.set_transient_for(self.dialog)
     self.distro = distro
@@ -121,12 +123,12 @@ class DialogMirror:
     self.treeview.set_model(self.model_sort)
     # the cell renderer for the mirror uri
     self.renderer_mirror = Gtk.CellRendererText()
-    self.renderer_mirror.connect('edited', 
-                                 self.on_edited_custom_mirror, 
+    self.renderer_mirror.connect('edited',
+                                 self.on_edited_custom_mirror,
                                  self.model)
     # the visible column that holds the mirror uris
-    self.column_mirror = Gtk.TreeViewColumn("URI", 
-                                            self.renderer_mirror, 
+    self.column_mirror = Gtk.TreeViewColumn("URI",
+                                            self.renderer_mirror,
                                             text=COLUMN_URI)
     self.treeview.append_column(self.column_mirror)
 
@@ -138,8 +140,8 @@ class DialogMirror:
     if len(self.custom_mirrors) > 0:
         for mirror in self.custom_mirrors:
             model.append(None, [mirror, False, True, None])
-            self.column_mirror.add_attribute(self.renderer_mirror, 
-                                             "editable", 
+            self.column_mirror.add_attribute(self.renderer_mirror,
+                                             "editable",
                                              COLUMN_CUSTOM)
         model.append(None, [None, True, False, None])
     # secondly add all official mirrors
@@ -149,7 +151,7 @@ class DialogMirror:
             model.append(map_loc[mirror.location],
                          [hostname, False, False, mirror])
         elif mirror.location != None:
-            parent = model.append(None, 
+            parent = model.append(None,
                                   [self.country_info.get_country_name(mirror.location), False, False, None])
             if mirror.location == self.country_info.code and patriot == None:
                 patriot = parent
@@ -173,7 +175,7 @@ class DialogMirror:
     iter_next = model.iter_next(iter)
     if new_text != "":
         model.set_value(iter, COLUMN_URI, new_text)
-        # Add a separator if the next mirror is a not a separator or 
+        # Add a separator if the next mirror is a not a separator or
         # a custom one
         if iter_next != None and not \
            (model.get_value(iter_next, COLUMN_SEPARATOR) or \
@@ -248,7 +250,7 @@ class DialogMirror:
     model.append(None, [_("New mirror"), False, True, None])
     self.treeview.grab_focus()
     self.treeview.set_cursor((0,),
-                             focus_column=self.column_mirror, 
+                             focus_column=self.column_mirror,
                              start_editing=True)
 
   def on_button_mirror_edit_clicked(self, button, data=None):
@@ -292,7 +294,7 @@ class DialogMirror:
         return None
 
   def on_button_test_clicked(self, button, data=None):
-    ''' Perform a test to find the best mirror and select it 
+    ''' Perform a test to find the best mirror and select it
         afterwards in the treeview '''
     self.button_cancel.set_sensitive(True)
     self.dialog_test.show()
@@ -331,7 +333,7 @@ class DialogMirror:
     if test.best != None:
         self.model_sort.foreach(self.select_mirror, test.best)
     else:
-        dialogs.show_error_dialog(self.dialog, 
+        dialogs.show_error_dialog(self.dialog,
                                   _("No suitable download server was found"),
                                   _("Please check your Internet connection."))
 
