@@ -20,12 +20,11 @@ import os
 from gettext import gettext as _
 import gi
 gi.require_version("Gtk", "3.0")
-from gi.repository import Gtk,GLib,Gio
+from gi.repository import Gtk,Gdk,GLib
 from softwareproperties.gtk.utils import setup_ui
 from uaclient.api.u.pro.attach.magic.initiate.v1 import initiate
 from uaclient.api.u.pro.attach.magic.wait.v1 import MagicAttachWaitOptions, wait
 from uaclient.exceptions import MagicAttachTokenError
-import threading
 
 class DialogUaAttach:
     def __init__(self, parent, datadir, ua_object):
@@ -39,13 +38,9 @@ class DialogUaAttach:
         self.contract_token = None
         self.attaching = False
         self.poll = None
-        self.pin = ""
+        self.pin_label_box.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(0.5, 0.5, 0.5, 0.5))
 
-        self.net_monitor = Gio.network_monitor_get_default()
-        self.net_monitor.connect("network-changed", self.net_status_changed, 0)
-        self.net_status_changed(
-            self.net_monitor, self.net_monitor.get_network_available(), 1
-        )
+        self.start_magic_attach()
 
     def run(self):
         self.dialog.run()
@@ -58,6 +53,7 @@ class DialogUaAttach:
         expired: called by the token polling when the token expires.
         """
         if self.token_radio.get_active():
+            self.pin_label.set_opacity(0)
             self.confirm.set_sensitive(self.token_field.get_text() != "" and
                                        not self.attaching)
             icon = self.token_status_icon
@@ -65,6 +61,7 @@ class DialogUaAttach:
             status = self.token_status
         else:
             self.pin_label.set_text(self.pin)
+            self.pin_label.set_opacity(1)
             self.confirm.set_sensitive(self.contract_token != None and
                                        not self.attaching)
             icon = self.pin_status_icon
@@ -84,16 +81,6 @@ class DialogUaAttach:
         self.token_field.set_sensitive(not self.attaching
                                        and self.token_radio.get_active())
 
-        # Unconditionally hide the "other radio section" icon/status.
-        # Show icon/status of the "current radio section" only if case is set.
-        self.token_status_icon.set_visible(False)
-        self.token_status.set_visible(False)
-        self.pin_status_icon.set_visible(False)
-        self.pin_status.set_visible(False)
-        if (case != None):
-            icon.set_visible(True)
-            status.set_visible(True)
-
         if (case == "fail"):
             status.set_markup('<span foreground="red">%s</span>' % _('Invalid token'))
             icon.set_from_icon_name('emblem-unreadable', 1)
@@ -107,6 +94,14 @@ class DialogUaAttach:
             status.set_markup(_('Code expired'))
             icon.set_from_icon_name('gtk-dialog-warning', 1)
 
+        #Only show icons/status if case is set
+        self.token_status_icon.set_visible(False)
+        self.token_status.set_visible(False)
+        self.pin_status_icon.set_visible(False)
+        self.pin_status.set_visible(False)
+        icon.set_visible(case)
+        status.set_visible(case)
+
     def attach(self):
         if self.attaching:
             return
@@ -140,6 +135,8 @@ class DialogUaAttach:
         self.attach()
 
     def on_cancel_clicked(self, button):
+        if self.poll:
+            GLib.Thread.unref(self.poll)
         self.dialog.response(Gtk.ResponseType.CANCEL)
 
     def poll_for_magic_token(self):
@@ -150,8 +147,6 @@ class DialogUaAttach:
             GLib.idle_add(self.update_state, 'pin_validated')
         except MagicAttachTokenError:
             GLib.idle_add(self.update_state, 'expired')
-        except Exception as e:
-            print("Error getting the Ubuntu Pro token: ", e, flush = True)
         finally:
             self.poll = None
 
@@ -170,11 +165,10 @@ class DialogUaAttach:
             self.pin = response.user_code
             self.req_id = response.token
         except Exception as e:
-            print("Error retrieving magic token: ", e)
+            print(e)
             return
         self.update_state()
-        self.poll = threading.Thread(target=self.poll_for_magic_token, daemon=True)
-        self.poll.start()
+        self.poll = GLib.Thread.new("poll", self.poll_for_magic_token)
 
     def on_radio_toggled(self, button):
         self.update_state()
@@ -182,20 +176,5 @@ class DialogUaAttach:
     def on_magic_radio_clicked(self, button):
         self.start_magic_attach()
 
-    # Do not control the radio buttons and confirm button widgets directly,
-    # since those former are controlled by update_state and this function must
-    # be logically independent of it. Control the net_control_box'es instead.
-    def net_status_changed(self, monitor, available, first_run):
-        self.no_connection.set_visible(not available)
-        self.radio_net_control_box.set_sensitive(available)
-        self.confirm_net_control_box.set_sensitive(available)
-        if available:
-            if self.pin == "":
-                self.start_magic_attach()
-            elif self.poll == None:
-                # wait() timed out without internet; Restart polling.
-                self.poll = threading.Thread(target=self.poll_for_magic_token, daemon=True)
-                self.poll.start()
-
     def finish(self):
         self.dialog.response(Gtk.ResponseType.OK)
