@@ -24,9 +24,6 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 #  USA
 
-from __future__ import absolute_import, print_function
-
-import apt
 import apt_pkg
 import datetime
 import dbus
@@ -44,9 +41,10 @@ import time
 import gi
 gi.require_version("Gdk", "3.0")
 gi.require_version("Gtk", "3.0")
+gi.require_version("Handy", "1")
 gi.require_version("PackageKitGlib", "1.0")
 from gi.repository import PackageKitGlib as packagekit
-from gi.repository import GObject, Gdk, Gtk, Gio, GLib
+from gi.repository import GObject, Gdk, Gtk, Gio, GLib, Handy
 
 from .SimpleGtkbuilderApp import SimpleGtkbuilderApp
 from .DialogAdd import DialogAdd
@@ -162,6 +160,11 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         SoftwareProperties.__init__(self, options=options, datadir=datadir)
         Gtk.Window.set_default_icon_name("software-properties")
 
+        # Support GNOME 42 dark mode
+        Handy.init()
+        style_manager = Handy.StyleManager.get_default()
+        style_manager.set_color_scheme(Handy.ColorScheme.PREFER_LIGHT)
+
         SimpleGtkbuilderApp.__init__(self,
                                      os.path.join(datadir, "gtkbuilder", "main.ui"),
                                      domain="software-properties")
@@ -224,7 +227,9 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         # used to store the handlers of callbacks
         self.handlers = {}
 
-        self.apt_cache = {}
+        # Initialise and store the apt cache
+        self.init_apt_cache()
+
         self.pk_task = None
 
         # Put some life into the user interface:
@@ -243,8 +248,9 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.show_distro()
         # Setup and show the Additonal Drivers tab
         self.init_drivers()
-        # Setup and show the Ubuntu Pro tab
-        self.init_ubuntu_pro()
+        # Setup and show the Ubuntu Pro tab if the serie is a LTS
+        if is_current_distro_lts():
+            self.init_ubuntu_pro()
 
         # Connect to switch-page before setting initial tab. Otherwise the
         # first switch goes unnoticed.
@@ -287,8 +293,19 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         i = self.get_release_upgrades_policy()
         self.combobox_release_upgrades.set_active(i)
         self.handlers[self.combobox_release_upgrades] = \
-             self.combobox_release_upgrades.connect("changed", 
+             self.combobox_release_upgrades.connect("changed",
                                                     self.on_combobox_release_upgrades_changed)
+    def init_apt_cache(self):
+        apt_pkg.init_config()
+        apt_pkg.init_system()
+        self.apt_cache = apt_pkg.Cache(None)
+        self.depcache = apt_pkg.DepCache(self.apt_cache)
+        self.records = apt_pkg.PackageRecords(self.apt_cache)
+
+    def update_apt_cache(self):
+        self.apt_cache = apt_pkg.Cache(None)
+        self.depcache = apt_pkg.DepCache(self.apt_cache)
+        self.records = apt_pkg.PackageRecords(self.apt_cache)
 
     def init_auto_update(self):
         """ Set up the widgets that allow to configure the update automation """
@@ -982,7 +999,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
     # dbus events
     def on_config_modified(self):
         """The config was changed and now needs to be saved and reloaded"""
-        apt.apt_pkg.init_config()
+        apt_pkg.init_config()
         self.button_revert.set_sensitive(True)
 
     def on_keys_modified(self):
@@ -1217,16 +1234,15 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             self.progress_bar.set_fraction(prog_value / 100.0)
 
     def on_driver_changes_finish(self, source, result, installs_pending):
-        results = None
         try:
-            results = self.pk_task.generic_finish(result)
+            self.pk_task.generic_finish(result)
         except Exception as e:
             self.on_driver_changes_revert()
             error(self.window_main, _("Error while applying changes"), str(e))
         if not installs_pending:
             self.progress_bar.set_visible(False)
             self.clear_changes()
-            self.apt_cache = apt.Cache()
+            self.update_apt_cache()
             self.set_driver_action_status()
             self.update_label_and_icons_from_status()
             self.button_driver_revert.set_visible(True)
@@ -1239,19 +1255,20 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         installs = []
         removals = []
 
+        has_nvidia = False
         for pkg in self.driver_changes:
-            if pkg.is_installed:
-                removals.append(self.get_package_id(pkg.installed))
+            if pkg.current_ver:
+                removals.append(self.get_package_id(self.apt_cache, pkg))
                 # The main NVIDIA package is only a metapackage.
                 # We need to collect its dependencies, so that
                 # we can uninstall the driver properly.
-                if 'nvidia' in pkg.shortname:
-                    for dep in self.get_dependencies(self.apt_cache, pkg.shortname, 'nvidia'):
-                        dep_pkg = self.apt_cache[dep]
-                        if dep_pkg.is_installed:
-                            removals.append(self.get_package_id(dep_pkg.installed))
+                if 'nvidia' in pkg.name:
+                    has_nvidia = True
+                    for dep in self.get_dependencies(self.apt_cache, pkg, 'nvidia'):
+                        if dep.current_ver:
+                            removals.append(self.get_package_id(self.apt_cache, dep))
             else:
-                installs.append(self.get_package_id(pkg.candidate))
+                installs.append(self.get_package_id(self.apt_cache, pkg))
 
         self.cancellable = Gio.Cancellable()
         try:
@@ -1269,6 +1286,11 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                             installs_pending  # callback data
                  )
             if installs:
+                if has_nvidia:
+                    to_install = []
+                    for item in installs:
+                        name = item.split(';')[0]
+                        to_install.append(name)
                 self.pk_task.install_packages_async(installs,
                         self.cancellable,  # cancellable
                         self.on_driver_changes_progress,
@@ -1376,7 +1398,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         # WARNING: This is run in a separate thread.
         self.detect_called = True
         try:
-            self.apt_cache = apt.Cache()
             self.devices = detect.system_device_drivers(self.apt_cache)
         except:
             # Catch all exceptions and feed them to apport.
@@ -1395,22 +1416,16 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             return
 
         pkg = None
-        try:
-            if pkg_name:
-                pkg = self.apt_cache[pkg_name]
-                # If the package depends on dkms
-                # we need to install the correct linux metapackage
-                # so that users get the latest headers
-                if 'dkms' in pkg.candidate.record['Depends']:
-                    linux_meta = detect.get_linux(self.apt_cache)
-                    if (linux_meta and
-                        linux_meta not in self.driver_changes):
-                        # Install the linux metapackage
-                        lmp = self.apt_cache[linux_meta]
-                        if not lmp.is_installed:
-                            self.driver_changes.append(lmp)
-        except (AttributeError, KeyError):
-            pass
+        if pkg_name:
+            pkg = self.apt_cache[pkg_name]
+            # Add the matching linux modules package when available
+            try:
+                modules_package = detect.get_linux_modules_metapackage(self.apt_cache, pkg_name)
+                modules_package_obj = self.apt_cache[modules_package]
+                if modules_package and not modules_package_obj.current_ver:
+                    self.driver_changes.append(modules_package_obj)
+            except (KeyError, TypeError):
+                pass
 
         if button.get_active():
             if pkg in self.driver_changes:
@@ -1430,7 +1445,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
 
             if (pkg is not None
                 and pkg not in self.driver_changes
-                    and pkg.is_installed):
+                    and pkg.current_ver):
                 self.driver_changes.append(pkg)
 
         self.button_driver_revert.set_sensitive(bool(self.driver_changes))
@@ -1492,11 +1507,14 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
 
             try:
                 pkg = self.apt_cache[pkg_driver_name]
-                installed = pkg.is_installed
-                if pkg.candidate is not None:
-                    description = _("Using {} from {}").format(pkg.candidate.summary, pkg.shortname)
+                installed = pkg.current_ver
+                candidate = self.depcache.get_candidate_ver(pkg)
+
+                if candidate is not None:
+                    self.records.lookup(candidate.file_list[0])
+                    description = _("Using {} from {}").format(self.records.short_desc, pkg_driver_name)
                 else:
-                    description = _("Using {}").format(pkg.shortname)
+                    description = _("Using {}").format(pkg_driver_name)
             except KeyError:
                 print("WARNING: a driver ({}) doesn't have any available package associated: {}".format(pkg_driver_name, current_driver))
                 continue
@@ -1626,7 +1644,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         for device in self.devices:
             for pkg_name in self.devices[device]['drivers']:
                 pkg = self.apt_cache[pkg_name]
-                if not self.devices[device]['drivers'][pkg_name]['free'] and pkg.is_installed:
+                if not self.devices[device]['drivers'][pkg_name]['free'] and pkg.current_ver:
                     self.nonfree_drivers = self.nonfree_drivers + 1
 
         if self.nonfree_drivers > 0:
