@@ -15,9 +15,9 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 #  USA
 
+import apt_pkg
 import os
 import re
-import apt_pkg
 import subprocess
 import tempfile
 
@@ -36,8 +36,11 @@ from gettext import gettext as _
 from urllib.parse import urlparse
 
 
+apt_pkg.init()
+
 GPG_KEYRING_CMD = 'gpg -q --no-options --no-default-keyring --batch --keyring %s'
 
+
 class ShortcutHandler(object):
     '''Superclass for shortcut handler implementations.
 
@@ -295,6 +298,10 @@ class ShortcutHandler(object):
             # If the file doesn't exist, create it so we can set the mode
             for entryfile in set([binentry.file, srcentry.file]):
                 if not os.path.exists(entryfile):
+                    # Create the dir if needed
+                    if (entryfile.startswith(self.sourceparts_path)
+                        and not os.path.exists(self.sourceparts_path)):
+                        os.mkdir(self.sourceparts_path, 0o755)
                     with open(entryfile, 'w'):
                         os.chmod(entryfile, mode)
             sourceslist.save()
@@ -402,9 +409,14 @@ class ShortcutHandler(object):
         cmd = GPG_KEYRING_CMD % dest
         action = "--import"
         if not self.dry_run:
-            if not os.path.exists(dest) and self.trustedparts_mode:
-                with open(dest, 'wb'):
-                    os.chmod(dest, self.trustedparts_mode)
+            if not os.path.exists(dest):
+                # Create the dir if needed
+                if (dest.startswith(self.trustedparts_path)
+                    and not os.path.exists(self.trustedparts_path)):
+                    os.mkdir(self.trustedparts_path, mode=0o755)
+                if self.trustedparts_mode:
+                    with open(dest, 'wb'):
+                        os.chmod(dest, self.trustedparts_mode)
             try:
                 with tempfile.TemporaryDirectory() as homedir:
                     cmd += f" --homedir {homedir} {action}"
@@ -503,9 +515,14 @@ class ShortcutHandler(object):
 
         print(_("Adding authentication data to %s") % dest)
         if not self.dry_run:
-            if newfile and self.netrcparts_mode:
-                with open(dest, 'w'):
-                    os.chmod(dest, self.netrcparts_mode)
+            if newfile:
+                # Create the dir if needed
+                if (dest.startswith(self.netrcparts_path)
+                    and not os.path.exists(self.netrcparts_path)):
+                    os.mkdir(self.netrcparts_path, mode=0o755)
+                if self.netrcparts_mode:
+                    with open(dest, 'w'):
+                        os.chmod(dest, self.netrcparts_mode)
             with open(dest, 'a') as f:
                 # we're appending; if the file doesn't end in \n, throw one in
                 if finalchar != '\n':
